@@ -152,34 +152,52 @@ Returns the TabForm object that will be used in generating the edit page for thi
 =cut
 
 sub getEditForm {
-	my $self = shift;
-	my $tabform = $self->SUPER::getEditForm();
-	my $i18n = WebGUI::International->new($self->session,"Asset_Image");
-        $tabform->getTab("properties")->integer(
-               	-name=>"thumbnailSize",
-		-label=>$i18n->get('thumbnail size'),
-		-hoverHelp=>$i18n->get('Thumbnail size description'),
-		-value=>$self->getValue("thumbnailSize")
-               	);
-	$tabform->getTab("properties")->textarea(
-		-name=>"parameters",
-		-label=>$i18n->get('parameters'),
-		-hoverHelp=>$i18n->get('Parameters description'),
-		-value=>$self->getValue("parameters")
-		);
-	if ($self->get("filename") ne "") {
-		$tabform->getTab("properties")->readOnly(
-			-label=>$i18n->get('thumbnail'),
-			-hoverHelp=>$i18n->get('Thumbnail description'),
-			-value=>'<a href="'.$self->getFileUrl.'"><img src="'.$self->getThumbnailUrl.'?noCache='.$self->session->datetime->time().'" alt="thumbnail" /></a>'
-			);
-		my ($x, $y) = $self->getStorageLocation->getSizeInPixels($self->get("filename"));
-        	$tabform->getTab("properties")->readOnly(
-			-label=>$i18n->get('image size'),
-			-value=>$x.' x '.$y
-			);
-	}
-	return $tabform;
+    my $self = shift;
+    my $tabform = $self->SUPER::getEditForm();
+
+# Add the fields defined locally and apply any overrides from the config file
+    my $i18n = WebGUI::International->new($self->session,"Asset_Image");
+
+    tie my %extraFields, "Tie::IxHash";
+
+    $extraFields{thumbnailSize} = {
+        fieldType => "integer",
+        name      => "thumbnailSize",
+        label     => $i18n->get('thumbnail size'),
+        hoverHelp => $i18n->get('Thumbnail size description'),
+        value     => $self->getValue("thumbnailSize"),
+    };
+    $extraFields{parameters} = {
+        fieldType => "textarea",
+        name      => "parameters",
+        label     => $i18n->get('parameters'),
+        hoverHelp => $i18n->get('Parameters description'),
+        value     => $self->getValue("parameters"),
+    };
+
+    if ($self->get("filename") ne "") {
+          my ($x, $y) = $self->getStorageLocation->getSizeInPixels($self->get("filename"));
+
+          $extraFields{thumbnail} = {
+              fieldType => "readOnly",
+              label     => $i18n->get('thumbnail'),
+              hoverHelp => $i18n->get('Thumbnail description'),
+              value     => '<a href="'.$self->getFileUrl.'"><img src="'.$self->getThumbnailUrl.'?noCache='.$self->session->datetime->time().'" alt="thumbnail" /></a>'
+          };
+          $extraFields{imageSize} = {
+              fieldType => "readOnly",
+              label     => $i18n->get('image size'),
+              value     => $x.' x '.$y,
+          };
+    }
+
+    my $overrides = $self->session->config->get("assets/".$self->get("className"));
+
+    foreach my $fieldName (keys %extraFields) {
+        $self->setupFormField($tabform, $fieldName, \%extraFields, $overrides);
+    }
+
+    return $tabform;
 }
 
 #-------------------------------------------------------------------
@@ -354,10 +372,10 @@ sub www_annotate {
 	$style->setLink($url->extras('yui/build/imagecropper/assets/skins/sam/imagecropper.css'), {rel=>'stylesheet', type=>'text/css'});
 
 	$style->setScript($url->extras('yui/build/yahoo-dom-event/yahoo-dom-event.js'), {type=>'text/javascript'});
-	$style->setScript($url->extras('yui/build/element/element-beta-min.js'), {type=>'text/javascript'});
+	$style->setScript($url->extras('yui/build/element/element-min.js'), {type=>'text/javascript'});
 	$style->setScript($url->extras('yui/build/dragdrop/dragdrop-min.js'), {type=>'text/javascript'});
 	$style->setScript($url->extras('yui/build/resize/resize-min.js'), {type=>'text/javascript'});
-	$style->setScript($url->extras('yui/build/imagecropper/imagecropper-beta-min.js'), {type=>'text/javascript'});
+	$style->setScript($url->extras('yui/build/imagecropper/imagecropper-min.js'), {type=>'text/javascript'});
 
 	my @pieces = split(/\n/, $self->get('annotations'));
 	
@@ -633,7 +651,7 @@ sub www_resize {
 	$style->setLink($url->extras('yui/build/fonts/fonts-min.css'), {rel=>'stylesheet', type=>'text/css'});
 	$style->setLink($url->extras('yui/build/resize/assets/skins/sam/resize.css'), {rel=>'stylesheet', type=>'text/css'});
 	$style->setScript($url->extras('yui/build/yahoo-dom-event/yahoo-dom-event.js'), {type=>'text/javascript'});
-	$style->setScript($url->extras('yui/build/element/element-beta-min.js'), {type=>'text/javascript'});
+	$style->setScript($url->extras('yui/build/element/element-min.js'), {type=>'text/javascript'});
 	$style->setScript($url->extras('yui/build/dragdrop/dragdrop-min.js'), {type=>'text/javascript'});
 	$style->setScript($url->extras('yui/build/resize/resize-min.js'), {type=>'text/javascript'});
 	$style->setScript($url->extras('yui/build/animation/animation-min.js'), {type=>'text/javascript'});
@@ -774,10 +792,10 @@ sub www_crop {
 	$style->setLink($url->extras('yui/build/fonts/fonts-min.css'), {rel=>'stylesheet', type=>'text/css'});
 	$style->setLink($url->extras('yui/build/imagecropper/assets/skins/sam/imagecropper.css'), {rel=>'stylesheet', type=>'text/css'});
 	$style->setScript($url->extras('yui/build/yahoo-dom-event/yahoo-dom-event.js'), {type=>'text/javascript'});
-	$style->setScript($url->extras('yui/build/element/element-beta-min.js'), {type=>'text/javascript'});
+	$style->setScript($url->extras('yui/build/element/element-min.js'), {type=>'text/javascript'});
 	$style->setScript($url->extras('yui/build/dragdrop/dragdrop-min.js'), {type=>'text/javascript'});
 	$style->setScript($url->extras('yui/build/resize/resize-min.js'), {type=>'text/javascript'});
-	$style->setScript($url->extras('yui/build/imagecropper/imagecropper-beta-min.js'), {type=>'text/javascript'});
+	$style->setScript($url->extras('yui/build/imagecropper/imagecropper-min.js'), {type=>'text/javascript'});
 
 	my $i18n = WebGUI::International->new($session,"Asset_Image");
 
