@@ -492,7 +492,7 @@ sub www_managePayouts {
     $style->setLink($url->extras('yui/build/button/assets/skins/sam/button.css'),       {type=>'text/css', rel=>'stylesheet'});
 
     $style->setScript($url->extras('yui/build/yahoo-dom-event/yahoo-dom-event.js'), {type=>'text/javascript'});
-    $style->setScript($url->extras('yui/build/element/element-beta-min.js'),        {type=>'text/javascript'});
+    $style->setScript($url->extras('yui/build/element/element-min.js'),             {type=>'text/javascript'});
     $style->setScript($url->extras('yui/build/connection/connection-min.js'),       {type=>'text/javascript'});
     $style->setScript($url->extras('yui/build/json/json-min.js'),                   {type=>'text/javascript'});
     $style->setScript($url->extras('yui/build/paginator/paginator-min.js'),         {type=>'text/javascript'});
@@ -687,10 +687,13 @@ sub www_vendorTotalsAsJSON {
     foreach my $vendorId (keys %{ $vendorPayoutData }) {
         my $vendor = WebGUI::Shop::Vendor->new( $session, $vendorId );
 
-        push @dataset, {
+        my $dataset = {
             %{ $vendor->get },
             %{ $vendorPayoutData->{ $vendorId } },
-        }
+        };
+        my $user = WebGUI::User->new($session, $vendor->get('userId'));
+        $dataset->{name} .= ' ('.$user->username.')';
+        push @dataset, $dataset;
     }
 
     $session->http->setMimeType( 'application/json' );
