@@ -954,21 +954,17 @@ sub isInGroup {
 	### Check stow before we check the cache.  Stow is in memory and much faster
 	my $stow          = $session->stow->get("isInGroup", { noclone => 1 }) || {};
 	return $stow->{$uid}->{$gid} if (exists $stow->{$uid}->{$gid});
-	
+
 	### Don't bother checking File Cache if we already have a stow for this group.
 	### We can find what we need there and save ourselves a bunch of time
-	my $cache        = undef;
-	my $groupMembers = undef;
-	unless ($stow->{$uid}->{$gid}) {
-		$cache         = WebGUI::Cache->new($session,["groupMembers",$gid]);
-		$groupMembers  = $cache->get || {};
-		#If we have this user's membership cached, return what we have stored
-		if (exists $groupMembers->{$uid}) {
-			return $groupMembers->{$uid}->{isMember} if (!$self->isVisitor);
-			return $groupMembers->{$uid}->{$session->getId}->{isMember} #Include the session check for visitors
-		}
-	}
-	
+    my $cache         = WebGUI::Cache->new($session,["groupMembers",$gid]);
+    my $groupMembers  = $cache->get || {};
+    #If we have this user's membership cached, return what we have stored
+    if (exists $groupMembers->{$uid}) {
+        return $groupMembers->{$uid}->{isMember} if (!$self->isVisitor);
+        return $groupMembers->{$uid}->{$session->getId}->{isMember} if exists $groupMembers->{$uid}->{$session->getId}; #Include the session check for visitors
+    }
+
  	### Instantiate the group
     my $group = WebGUI::Group->new($session,$gid);
 	if ( !$group ) {
@@ -978,7 +974,7 @@ sub isInGroup {
 
 	#Check the group for membership
 	my $isInGroup = $group->hasUser($self);
-	
+
 	#Write what we found to file cache
 	$group->cacheGroupings( $self, $isInGroup, $cache, $groupMembers );
 	return $isInGroup;
@@ -1422,27 +1418,28 @@ sub update {
     delete $properties->{wg_privacySettings};
 
     # $self->{_user} contains all fields in `users` table
-    my @userFields  = ();
-    my @userValues  = ();
+    my @userFields = ();
+    my @userValues = ();
     for my $key ( keys %{$self->{_user}} ) {
         if ( exists $properties->{$key} ) {
             # Delete the value because it's not a profile field
-            my $value   = delete $properties->{$key};
+            my $value = delete $properties->{$key};
             push @userFields, $db->dbh->quote_identifier( $key ) . " = ?";
             push @userValues, $value;
             $self->{_user}->{$key} = $value;
         }
     }
     # No matter what we update properties
-    my $userFields  = join ", ", @userFields;
+    my $userFields = join ", ", @userFields;
     $db->write(
         "UPDATE users SET $userFields WHERE userId=?",
         [@userValues, $self->{_userId}]
     );
 
     # Everything else must be a profile field
-    my @profileFields   = ();
-    my @profileValues   = ();
+    my @profileFields = ();
+    my @profileValues = ();
+
     for my $key ( keys %{$properties} ) {
         if (!exists $self->{_profile}{$key} && !WebGUI::ProfileField->exists($session,$key)) {
             $self->session->errorHandler->warn("No such profile field: $key");
@@ -1453,7 +1450,7 @@ sub update {
         $self->{_profile}->{$key} = $properties->{ $key };
     }
     if ( @profileFields ) {
-        my $profileFields  = join ", ", @profileFields;
+        my $profileFields = join ", ", @profileFields;
         $db->write(
             "UPDATE userProfileData SET $profileFields WHERE userId=?",
             [@profileValues, $self->{_userId}]
