@@ -2,8 +2,24 @@ package WebGUI::i18n::German::Asset_Carousel;
 use utf8;
 our $I18N = {
   'assetName' => {
-    'lastUpdated' => '1244064122',
-    'message' => 'Carousel'
+    'lastUpdated' => '1301988649',
+    'message' => 'Karussell'
+  },
+  'carousel autoPlay description' => {
+    'lastUpdated' => '1301988620',
+    'message' => 'Soll dieses Karussell automatisch durch die Einträge scrollen?'
+  },
+  'carousel autoPlay label' => {
+    'lastUpdated' => '1301988661',
+    'message' => 'Auto Play'
+  },
+  'carousel autoPlayInterval description' => {
+    'lastUpdated' => '1301988714',
+    'message' => 'Länge in Sekunden zwischen den Karussell-Wechseln'
+  },
+  'carousel autoPlayInterval label' => {
+    'lastUpdated' => '1301988729',
+    'message' => 'Auto-Play Intervall'
   },
   'carousel slideHeight description' => {
     'lastUpdated' => '1281649002',
@@ -33,6 +49,10 @@ our $I18N = {
     'lastUpdated' => '1244064459',
     'message' => 'Carouselvorlage'
   },
+  'delete' => {
+    'lastUpdated' => '1295596758',
+    'message' => 'Löschen'
+  },
   'id description' => {
     'lastUpdated' => '1244064296',
     'message' => 'Geben Sie eine eindeutige ID für dieses Carouselitem ein.'
@@ -57,6 +77,10 @@ our $I18N = {
     'lastUpdated' => '1244064335',
     'message' => 'Items'
   },
+  'rich editor description' => {
+    'lastUpdated' => '1303289466',
+    'message' => 'Wählen Sie einen Komfort-Editor für die Inhaltseingabe in die Carousel-Bereiche. Diese Einstellung wird bei der nächsten Carousel-Nutzung wirksam.'
+  },
   'sequenceNumber' => {
     'lastUpdated' => '1244064369',
     'message' => 'Folgenummer für dieses Carouselitem.'
